#include "Crystalino.h"

// Define el LED RGB NEOPIXEL
Adafruit_NeoPixel led_rgb = Adafruit_NeoPixel(NUM_LEDS, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);

// Mando a distancia IR
const int RECV_PIN = RECEPTOR_IR;
IRrecv irrecv(RECV_PIN);
decode_results results;

int _estado_mando_ir = MANDO_IR_OFF;

void ini_crystalino()
{
	_ini_crystalino();
}

void ini_crystalino(int estado_ir)
{
	_ini_crystalino();
	
	if (estado_ir == MANDO_IR_ON)
	{
		// Inicializa el receptor del control remoto IR
		irrecv.enableIRIn();
		irrecv.blink13(true);
		_estado_mando_ir = MANDO_IR_ON;
	}
}

void _ini_crystalino()
{
  //Driver de motores DRV8833
  pinMode(MDER_1, OUTPUT);
  pinMode(MDER_2, OUTPUT);
  pinMode(MIZQ_1, OUTPUT);
  pinMode(MIZQ_2, OUTPUT);
  
  digitalWrite(MDER_1, LOW);
  digitalWrite(MDER_2, LOW);
  digitalWrite(MIZQ_1, LOW);
  digitalWrite(MIZQ_2, LOW);

  //Pulsador
  pinMode(PULSADOR, INPUT_PULLUP);
  
  //Altavoz
  pinMode(ALTAVOZ, OUTPUT);
  
  //Sensor de distancias por ultrasonidos HC-SR04
  pinMode (PIN_TRIG, OUTPUT);
  pinMode (PIN_ECHO, INPUT);

  //Led RGB NEOPIXEL
  led_rgb.begin();
  muestra_rgb(AZUL);
}

void mueve_motor(int vel, int motor)
{
  int pwm;
  int pin_1, pin_2;

  vel = constrain(vel, -100, 100);
  pwm = map(abs(vel), 0, 100, 0, 255);

  if (motor == MOTOR_DER)
  {
    pin_1 = MDER_1;
    pin_2 = MDER_2;
  }
  else if (motor == MOTOR_IZQ)
  {
    pin_1 = MIZQ_1;
    pin_2 = MIZQ_2;
  }
  else
    return;

  if (vel != 0)
  {
	if (vel > 0)
	{
		digitalWrite(pin_1, LOW);
		_estado_mando_ir == MANDO_IR_OFF ? analogWrite(pin_2, pwm) : digitalWrite(pin_2, HIGH);
	}
	else
	{
		_estado_mando_ir == MANDO_IR_OFF ? analogWrite(pin_1, pwm) : digitalWrite(pin_1, HIGH);
		digitalWrite(pin_2, LOW);
	}
  }
  else
  {
    digitalWrite(pin_1, HIGH);
    digitalWrite(pin_2, HIGH);
  }
}

void mueve_crystalino(int vel)
{
  constrain(vel, -100, 100);
  mueve_motor(vel, MOTOR_DER);
  mueve_motor(vel, MOTOR_IZQ);
}

void gira_crystalino(int vel)
{
  constrain(vel, -100, 100);
  mueve_motor(-vel, MOTOR_DER);
  mueve_motor(vel, MOTOR_IZQ);
}

void para_crystalino()
{
  mueve_motor(0, MOTOR_DER);
  mueve_motor(0, MOTOR_IZQ);
}

void espera_pulsado()
{
  while(digitalRead(PULSADOR) == HIGH);
  delay(100);
  while(digitalRead(PULSADOR) == LOW);
  delay(100);
}

void beep()
{
  sonido(800, 100);
}

void doble_beep()
{
  beep();
  delay(100);
  beep();
}

void triple_beep()
{
  beep();
  delay(100);
  beep();
  delay(100);
  beep();
}


void muestra_rgb(int color)
{
	switch (color)
	{
		case ROJO:
			led_rgb.setPixelColor(0, led_rgb.Color(255,0,0));
			break;
			
		case AMARILLO:
			led_rgb.setPixelColor(0, led_rgb.Color(255,255,0));
			break;
			
		case CIAN:
			led_rgb.setPixelColor(0, led_rgb.Color(0,255,255));
			break;
			
		case MAGENTA:
			led_rgb.setPixelColor(0, led_rgb.Color(255,0,255));
			break;
			
		case VERDE:
			led_rgb.setPixelColor(0, led_rgb.Color(0,255,0));
			break;
			
		case AZUL:
			led_rgb.setPixelColor(0, led_rgb.Color(0,0,255));
			break;
			
		case BLANCO:
			led_rgb.setPixelColor(0, led_rgb.Color(255,255,255));
			break;
			
		case NEGRO:
		default:
			led_rgb.setPixelColor(0, led_rgb.Color(0,0,0));
			break;
	}
	
	led_rgb.show();
}

int distancia_crystalino()
{
	int medida;
  
	
	//Realizamos medida del sensor de US
	//Generamos pulso de emision
	digitalWrite(PIN_TRIG, LOW);
	delayMicroseconds(10);
	digitalWrite(PIN_TRIG, HIGH);
	delayMicroseconds(10);
	digitalWrite(PIN_TRIG,LOW);

	// Medimos en cm, como máximo 255 cm
	medida = pulseIn(PIN_ECHO, HIGH, 20000) / 58;

	// Ajustamos la medida al rango máximo
	if (medida > 255)
		medida = 255;
  
	//El ultrasonidos devuelve un 0 cuando no se recibe ningun eco 
	//Cuando esto ocurre lo tratamos como rango maximo
	if (medida == 0)
		medida = 255;
  
   
	return medida;
}

void sonido(int frecuencia, int duracion)
{
  float periodo = 1000000 / frecuencia;
  long veces = duracion / (periodo/1000);
  int espera = (periodo) / 2;

  /*Serial.print(periodo);
  Serial.print(" ");
  Serial.print(veces);
  Serial.print(" ");
  Serial.println(espera);*/
  
  for (int x = 0; x < veces; x++)
  {
    digitalWrite(ALTAVOZ, HIGH);
    delayMicroseconds(espera);
    digitalWrite(ALTAVOZ, LOW);
    delayMicroseconds(espera);
  }
}

int comunica_mando_ir()
{
	return (irrecv.decode(&results));
}

unsigned long dato_mando_ir()
{
	static unsigned long ultimo_dato = 0;
	unsigned long dato = results.value;
	
	// Detecta pulsación mantenida
    if (dato == 0XFFFFFFFF)
      dato = ultimo_dato;
	else
	  ultimo_dato = dato;
  
	irrecv.resume();
  
	return dato;
}
